/*
	MIT License http://www.opensource.org/licenses/mit-license.php
	Author Tobias Koppers @sokra
*/
var ConcatSource = require("webpack-core/lib/ConcatSource");
var Template = require("./Template");

function AmdMainTemplatePlugin(name) {
	this.name = name;
}
module.exports = AmdMainTemplatePlugin;
AmdMainTemplatePlugin.prototype.apply = function(mainTemplate) {
	mainTemplate.plugin("render", function(source, chunk, hash, moduleTemplate, dependencyTemplates) {
		var externals = chunk.modules.filter(function(m) {
			return m.external;
		});
		var externalsDepsArray = JSON.stringify(externals.map(function(m) {
			return typeof m.request === "object" ? m.request.amd : m.request;
		}));
		var externalsArguments = externals.map(function(m) {
			return "__WEBPACK_EXTERNAL_MODULE_" + m.id + "__";
		}).join(", ");
		if(this.name) {
			var name = this.name
				.replace(Template.REGEXP_HASH, hash)
				.replace(Template.REGEXP_CHUNKHASH, chunk.renderedHash)
				.replace(Template.REGEXP_ID, chunk.id)
				.replace(Template.REGEXP_NAME, chunk.name || chunk.id);
			return new ConcatSource("define(" + JSON.stringify(name) + ", " + externalsDepsArray + ", function(" + externalsArguments + ") { return ", source, "});");
		} else if(externalsArguments) {
			return new ConcatSource("define(" + externalsDepsArray + ", function(" + externalsArguments + ") { return ", source, "});");
		} else {
			return new ConcatSource("define(function() { return ", source, "});");
		}
	}.bind(this));
	mainTemplate.plugin("global-hash", function(chunk) {
		if(Template.REGEXP_HASH.test(this.name || ""))
			return true;
	}.bind(this));
	mainTemplate.plugin("hash", function(hash) {
		hash.update("exports amd");
		hash.update(this.name + "");
	}.bind(this));
};